package cn.dg32z.lon.checks.util.processor.clickprocessor;

import cn.dg32z.libs.com.github.retrooper.packetevents.event.PacketReceiveEvent;
import cn.dg32z.libs.com.github.retrooper.packetevents.protocol.packettype.PacketType;
import cn.dg32z.libs.com.github.retrooper.packetevents.protocol.player.ClientVersion;
import cn.dg32z.libs.com.github.retrooper.packetevents.wrapper.play.client.WrapperPlayClientPlayerFlying;
import cn.dg32z.lon.PluginLoader;
import cn.dg32z.lon.checks.Check;
import cn.dg32z.lon.checks.type.PacketCheck;
import cn.dg32z.lon.checks.util.exempts.types.ExemptType;
import cn.dg32z.lon.player.PlayerData;
import cn.dg32z.lon.utils.evicting.EvictingList;
import java.util.Iterator;

/* loaded from: cn_dg32z_lon_checks_util_processor_clickprocessor.jar:cn/dg32z/lon/checks/util/processor/clickprocessor/ClickProcessor.class */
public final class ClickProcessor extends Check implements PacketCheck {
    public final EvictingList<Long> samples;
    public int cps;
    public int lastCps;
    public int swings;
    public long lastSwing;
    public long delay;
    private int done;
    private double kurtosis;

    public void setCps(int cps) {
        this.cps = cps;
    }

    public void setLastCps(int lastCps) {
        this.lastCps = lastCps;
    }

    public void setSwings(int swings) {
        this.swings = swings;
    }

    public void setLastSwing(long lastSwing) {
        this.lastSwing = lastSwing;
    }

    public void setDelay(long delay) {
        this.delay = delay;
    }

    public void setDone(int done) {
        this.done = done;
    }

    public void setKurtosis(double kurtosis) {
        this.kurtosis = kurtosis;
    }

    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ClickProcessor)) {
            return false;
        }
        ClickProcessor other = (ClickProcessor) o;
        if (!other.canEqual(this) || getCps() != other.getCps() || getLastCps() != other.getLastCps() || getSwings() != other.getSwings() || getLastSwing() != other.getLastSwing() || getDelay() != other.getDelay() || getDone() != other.getDone() || Double.compare(getKurtosis(), other.getKurtosis()) != 0) {
            return false;
        }
        Object this$samples = getSamples();
        Object other$samples = other.getSamples();
        return this$samples == null ? other$samples == null : this$samples.equals(other$samples);
    }

    protected boolean canEqual(Object other) {
        return other instanceof ClickProcessor;
    }

    public int hashCode() {
        int result = (1 * 59) + getCps();
        int result2 = (((result * 59) + getLastCps()) * 59) + getSwings();
        long $lastSwing = getLastSwing();
        int result3 = (result2 * 59) + ((int) (($lastSwing >>> 32) ^ $lastSwing));
        long $delay = getDelay();
        int result4 = (((result3 * 59) + ((int) (($delay >>> 32) ^ $delay))) * 59) + getDone();
        long $kurtosis = Double.doubleToLongBits(getKurtosis());
        int result5 = (result4 * 59) + ((int) (($kurtosis >>> 32) ^ $kurtosis));
        Object $samples = getSamples();
        return (result5 * 59) + ($samples == null ? 43 : $samples.hashCode());
    }

    public EvictingList<Long> getSamples() {
        return this.samples;
    }

    public int getCps() {
        return this.cps;
    }

    public int getLastCps() {
        return this.lastCps;
    }

    public int getSwings() {
        return this.swings;
    }

    public long getLastSwing() {
        return this.lastSwing;
    }

    public long getDelay() {
        return this.delay;
    }

    public int getDone() {
        return this.done;
    }

    public double getKurtosis() {
        return this.kurtosis;
    }

    public ClickProcessor(PlayerData player) {
        super(player);
        this.samples = new EvictingList<>(20);
        this.lastSwing = time();
    }

    public void onPacketReceive(PacketReceiveEvent event) {
        if (WrapperPlayClientPlayerFlying.isFlying(event.getPacketType())) {
            int needTick = this.player.getClientVersion().isOlderThanOrEquals(ClientVersion.V_1_8) ? 20 : 19;
            int i = this.done;
            this.done = i + 1;
            if (i >= needTick) {
                if (isExempt(new ExemptType[]{ExemptType.LAGGING, ExemptType.TELEPORT}) || this.player.getPacketStateData().lastPacketWasOnePointSeventeenDuplicate) {
                    this.done = 0;
                    this.swings = 0;
                }
                this.cps = this.swings;
                if (time() - this.lastSwing <= 350) {
                    this.lastCps = this.cps;
                }
                this.done = 0;
                this.swings = 0;
                return;
            }
            return;
        }
        if (event.getPacketType() == PacketType.Play.Client.ANIMATION && !isExempt(new ExemptType[]{ExemptType.INTERACT, ExemptType.LAGGING, ExemptType.PLACING}) && PluginLoader.INSTANCE.getConfigManager().getConfig().getString("function.click-listener.mode").contains("packet")) {
            this.swings++;
            long now = time();
            if (this.lastSwing > 0) {
                this.delay = now - this.lastSwing;
                this.samples.add(Long.valueOf(this.delay));
            }
            this.lastSwing = now;
        }
    }

    public double calculateDeviation() {
        double mean = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        return Math.sqrt(this.samples.stream().mapToDouble(l2 -> {
            return Math.pow(l2.longValue() - mean, 2.0d);
        }).average().orElse(0.0d));
    }

    public double calculateVariance() {
        double mean = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        return this.samples.stream().mapToDouble(l2 -> {
            return Math.pow(l2.longValue() - mean, 2.0d);
        }).average().orElse(0.0d);
    }

    public double calculateKurtosis() {
        double mean = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        double n = this.samples.size();
        double sumFourthMoment = this.samples.stream().mapToDouble(l2 -> {
            return Math.pow(l2.longValue() - mean, 4.0d);
        }).sum();
        return (n * sumFourthMoment) / Math.pow(calculateVariance(), 2.0d);
    }

    public double calculateSkewness() {
        double mean = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        double n = this.samples.size();
        double sumThirdMoment = this.samples.stream().mapToDouble(l2 -> {
            return Math.pow(l2.longValue() - mean, 3.0d);
        }).sum();
        return (n * sumThirdMoment) / Math.pow(calculateDeviation(), 3.0d);
    }

    public boolean checkClickConsistency(double threshold) {
        if (this.samples.size() < 5) {
            return false;
        }
        double mean = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        long withinThreshold = this.samples.stream().filter(l2 -> {
            return Math.abs(((double) l2.longValue()) - mean) < 5.0d;
        }).count();
        double consistency = withinThreshold / this.samples.size();
        return consistency >= threshold;
    }

    public boolean hasRepeatedPattern(int threshold) {
        if (this.samples.size() < 3) {
            return false;
        }
        Long first = (Long) this.samples.get(0);
        int repeatCount = 1;
        for (int i = 1; i < this.samples.size(); i++) {
            if (Math.abs(((Long) this.samples.get(i)).longValue() - first.longValue()) < 3) {
                repeatCount++;
                if (repeatCount >= threshold) {
                    return true;
                }
            } else {
                first = (Long) this.samples.get(i);
                repeatCount = 1;
            }
        }
        return false;
    }

    public boolean hasClickFrequencyTrend(int trendThreshold) {
        if (this.samples.size() < 3) {
            return false;
        }
        double trendScore = 0.0d;
        for (int i = 1; i < this.samples.size(); i++) {
            trendScore += Math.abs(((Long) this.samples.get(i)).longValue() - ((Long) this.samples.get(i - 1)).longValue());
        }
        return trendScore < ((double) trendThreshold);
    }

    public boolean hasSmallIntervalRange(int rangeThreshold) {
        if (this.samples.size() < 2) {
            return false;
        }
        long minInterval = Long.MAX_VALUE;
        long maxInterval = Long.MIN_VALUE;
        Iterator it = this.samples.iterator();
        while (it.hasNext()) {
            long sample = ((Long) it.next()).longValue();
            if (sample < minInterval) {
                minInterval = sample;
            }
            if (sample > maxInterval) {
                maxInterval = sample;
            }
        }
        return maxInterval - minInterval < ((long) rangeThreshold);
    }

    public boolean hasClickSpeedSurge(int surgeThreshold) {
        if (this.samples.size() < 5) {
            return false;
        }
        double averageSpeed = this.samples.stream().mapToLong(l -> {
            return l.longValue();
        }).average().orElse(0.0d);
        double speedChange = 0.0d;
        Iterator it = this.samples.iterator();
        while (it.hasNext()) {
            long sample = ((Long) it.next()).longValue();
            speedChange += Math.abs(sample - averageSpeed);
        }
        return speedChange > ((double) surgeThreshold);
    }
}
